import { Component, OnInit } from '@angular/core'
import { ApiService, Statistics } from '../services/api.service'
import { NzMessageService } from 'ng-zorro-antd/message'
import { NzModalService } from 'ng-zorro-antd/modal'
import { Chart } from 'chart.js'

@Component({
  selector: 'app-dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.scss']
})
export class DashboardComponent implements OnInit {
  protected statistics?: Statistics;
  constructor(
    private _messageService: NzMessageService,
    private _apiService: ApiService,
    public dialog: NzModalService
  ) {}
  ngOnInit() {
    this._apiService.getStatistics().subscribe(statistics => {this.statistics = statistics; this.renderChart();})
  }

  renderChart() {
    // Ensure a canvas is available before rendering the chart.
    const canvas = document.querySelector('canvas');
    if (!canvas) return;
    const ctx = canvas.getContext('2d');
    if (!ctx) return;

    new Chart(ctx, {
      type: 'line',
      data: {
        labels: this.statistics?.statement_history.map((_, i) => `Day ${i + 1}`),
        datasets: [{
          label: 'Statements Over Time',
          data: this.statistics?.statement_history,
          borderColor: 'blue',
          backgroundColor: 'rgba(0, 0, 255, 0.1)',
          fill: true,
        }]
      },
      options: {
        responsive: true,
        maintainAspectRatio: false,
        scales: {
          y: {
            beginAtZero: true
          }
        }
      }
    });
  }
}
